package bonsai

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Record.Path_orderSource

Path order is one of the scarier parts of the API and is explained further in the explanation for S.path_order.

Parameters

Signature

Sourcetype 'a t' =
  1. | [] : unit t'
  2. | :: : 'a M.t * 'b t' -> ('a -> 'b) t'
Sourcetype t =
  1. | T : 'a t' -> t