package bonsai

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Vdom_node_with_map_childrenSource

Sourceval make : tag:string -> ?attr:Bonsai_web.Vdom.Attr.t -> (_, Bonsai_web.Vdom.Node.t, _) Core.Map.t -> Bonsai_web.Vdom.Node.t

When given a map of vdom nodes, this function will wrap them in an element whose tag is determined by the first argument, and efficiently diff them against new nodes in the future that were created by this function.