package bonsai

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Bonsai_web_ui_formSource

Sourcemodule With_manual_view : sig ... end

With_manual_view forms are now the recommended way to build forms in Bonsai. You get full control over how you'd like to combine them. Historically, With_automatic_view was the default, but they are quite restrictive and hard to customize, so we recommend choosing With_manual_view for new forms.

Sourcemodule With_automatic_view : sig ... end

With_automatic_view forms have their views composed in an opinionated way. This used to be the default and recommended way to build forms in Bonsai, but now people should prefer With_manual_view where possible.