package bonsai

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Bonsai_web_ui_form.TypedSource

The functions in this module can be hard to understand Please look at the examples in lib/bonsai/examples/forms/typed.ml

Sourcemodule Record : sig ... end
Sourcemodule Variant : sig ... end