package bonsai

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module type Record.SSource

val label_for_field : [ `Inferred | `Computed of 'a Typed_field.t -> string | `Dynamic of (Typed_field.Packed.t -> string) Bonsai_web.Value.t ]

The label to use for each field of your record. If this value is `Inferred, each field will be labelled with its name. If this value is `Computed f, then each field will be labelled with the result of calling f on it.

val form_for_field : 'a Typed_field.t -> 'a Bonsai_web_ui_form__.Form.t Bonsai_web.Computation.t

For each of the fields in your record, you need to provide a form component which produces values of that type.