package bonsai

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Raw.TableSource

A lower-level module for building tables that puts more burden on the user to manage the structure of the table, but also provides more freedom regarding the structure of the table.

Sourceval header_row : ?attrs:Virtual_dom.Vdom.Attr.t list -> Header_cell.t list -> Header_row.t
Sourceval data_row : ?attrs:Virtual_dom.Vdom.Attr.t list -> Data_cell.t list -> Data_row.t
Sourceval table : Theme.t -> ?table_attrs:Virtual_dom.Vdom.Attr.t list -> header_rows:Header_row.t list -> data_rows:Data_row.t list -> unit -> Virtual_dom.Vdom.Node.t