package bonsai

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Bonsai.PrivateSource

Sourceval reveal_value : 'a Value.t -> 'a Private_value.t
Sourceval conceal_value : 'a Private_value.t -> 'a Value.t
Sourceval reveal_computation : 'a Computation.t -> 'a Private_computation.packed
Sourceval conceal_computation : 'a Private_computation.packed -> 'a Computation.t
Sourceval path : Bonsai__.Path.t Computation.t
Sourcemodule Value = Private_value
Sourcemodule Computation = Private_computation
Sourcemodule Apply_action : sig ... end
Sourcemodule Environment : sig ... end
Sourcemodule Meta : sig ... end

Meta.Model and Meta.Action contain the necessary functions associated with the hidden model and action types of Bonsai components. These are stored in Computation.packed.

Sourcemodule Snapshot : sig ... end

A snapshot represents the state of a component at an instant in time.

Sourcemodule Lifecycle : sig ... end
Sourcemodule Path : sig ... end
Sourcemodule Node_path : sig ... end
Sourcemodule Graph_info : sig ... end
Sourcemodule Instrumentation : sig ... end
Sourcemodule Flatten_values : sig ... end
Sourceval eval : environment:Environment.t -> path:Path.t -> clock:Ui_incr.Clock.t -> model:'model Ui_incr.t -> inject_dynamic:('dynamic_action -> unit Effect.t) -> inject_static:('static_action -> unit Effect.t) -> ('model, 'dynamic_action, 'static_action, 'result) Computation.t -> ('model, 'dynamic_action, 'result) Snapshot.t