package bonsai

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Source file test_action_stabilization.ml

1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
481
482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
507
508
509
510
511
512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
528
529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
549
550
551
552
553
554
555
556
557
558
559
560
561
562
563
564
565
566
567
568
569
570
571
572
573
574
575
576
577
578
579
580
581
582
583
584
585
586
587
588
589
590
591
592
593
594
595
596
597
598
599
600
601
602
603
604
605
606
607
608
609
610
611
612
613
614
615
616
617
618
619
620
621
622
623
624
625
626
627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
667
668
669
670
671
672
673
674
675
676
677
678
679
680
681
682
683
684
685
686
687
688
689
690
691
692
693
694
695
696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
731
732
733
734
735
736
737
738
739
740
741
742
743
744
745
746
747
748
749
750
751
752
753
754
755
756
757
758
759
760
761
762
763
764
765
766
767
768
769
770
771
772
773
774
775
776
777
778
779
780
781
782
783
784
785
786
787
788
789
790
791
792
793
794
795
796
797
798
799
800
801
802
803
804
805
806
807
808
809
810
811
812
813
814
815
816
817
818
819
820
821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
846
847
848
849
850
851
852
853
854
855
856
857
858
859
860
861
862
863
864
865
866
867
868
869
870
871
872
873
874
875
876
877
878
879
880
881
882
883
884
885
886
887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
918
919
920
921
922
923
924
925
926
927
928
929
930
931
932
933
934
935
936
937
938
939
940
941
942
943
944
945
946
947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
964
965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
983
984
985
986
987
988
989
990
991
992
993
994
995
996
997
998
999
1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
1015
1016
1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
1028
1029
1030
1031
1032
1033
1034
1035
1036
1037
1038
1039
1040
1041
1042
1043
1044
1045
1046
1047
1048
1049
1050
1051
1052
1053
1054
1055
1056
1057
1058
1059
1060
1061
1062
1063
1064
1065
1066
1067
1068
1069
1070
1071
1072
1073
1074
1075
1076
1077
1078
1079
1080
1081
1082
1083
1084
1085
1086
1087
1088
1089
1090
1091
1092
1093
1094
1095
1096
1097
1098
1099
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
1111
1112
1113
1114
1115
1116
1117
1118
1119
1120
1121
1122
1123
1124
1125
1126
1127
1128
1129
1130
1131
1132
1133
1134
1135
1136
1137
1138
1139
1140
1141
1142
1143
1144
1145
1146
1147
1148
1149
1150
1151
1152
1153
1154
1155
1156
1157
1158
1159
1160
1161
1162
1163
1164
1165
1166
1167
1168
1169
1170
1171
1172
1173
1174
1175
1176
1177
1178
1179
1180
1181
1182
1183
1184
1185
1186
1187
1188
1189
1190
1191
1192
1193
1194
1195
1196
1197
1198
1199
1200
1201
1202
1203
1204
1205
1206
1207
1208
1209
1210
1211
1212
1213
1214
1215
1216
1217
1218
1219
1220
1221
1222
1223
1224
1225
1226
1227
1228
1229
1230
1231
1232
1233
1234
1235
1236
1237
1238
1239
1240
1241
1242
1243
1244
1245
1246
1247
1248
1249
1250
1251
1252
1253
1254
1255
1256
1257
1258
1259
1260
1261
1262
1263
1264
1265
1266
1267
1268
1269
1270
1271
1272
1273
1274
1275
1276
1277
1278
1279
1280
1281
1282
1283
1284
1285
1286
1287
1288
1289
1290
1291
1292
1293
1294
1295
1296
1297
1298
1299
1300
1301
1302
1303
1304
1305
1306
1307
1308
1309
1310
1311
1312
1313
1314
1315
1316
1317
1318
1319
1320
1321
1322
1323
1324
1325
1326
1327
1328
1329
1330
1331
1332
1333
1334
1335
1336
1337
1338
open! Core
open! Import
open Bonsai.For_open
open Bonsai.Let_syntax
open Bonsai_test

let no_op_sm0 =
  Bonsai.state_machine0 ~apply_action:(fun _context () () -> ()) ~default_model:() ()
;;

let no_op_sm1 =
  Bonsai.state_machine1
    (opaque_const_value ())
    ~apply_action:(fun _context _input () () -> ())
    ~default_model:()
;;

let copying_sm1 input =
  Bonsai.state_machine1
    ~default_model:0
    ~apply_action:(fun _context input _ () ->
      match input with
      | Bonsai.Computation_status.Inactive -> failwith "impossible"
      | Bonsai.Computation_status.Active input -> input)
    input
;;

let%expect_test "Sub/Leaf1/Leaf0" =
  let component =
    let%sub _, inject1 = no_op_sm0 in
    let%sub _, inject2 = no_op_sm1 in
    let%sub _, inject3 = no_op_sm0 in
    let%sub _, inject4 = no_op_sm1 in
    let%arr inject1 = inject1
    and inject2 = inject2
    and inject3 = inject3
    and inject4 = inject4 in
    inject1 (), inject2 (), inject3 (), inject4 ()
  in
  let module Action = struct
    type t =
      | One
      | Two
      | Three
      | Four
    [@@deriving sexp_of]
  end
  in
  let handle =
    Handle.create
      (module struct
        type t = unit Effect.t * unit Effect.t * unit Effect.t * unit Effect.t
        type incoming = Action.t

        let view _ = ""

        let incoming (i1, i2, i3, i4) = function
          | Action.One -> i1
          | Two -> i2
          | Three -> i3
          | Four -> i4
        ;;
      end)
      component
  in
  Handle.print_actions handle;
  Handle.print_stabilizations handle;
  Handle.do_actions handle [ One; Two; Three; Four ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action" (action (Sub_from (Sub_from (Leaf_static <opaque>)))))
    stabilized
    ("Processed action" (action (Sub_from (Sub_into (Leaf_dynamic <opaque>)))))
    skipped stabilization
    ("Processed action" (action (Sub_into (Sub_from (Leaf_static <opaque>)))))
    stabilized
    ("Processed action" (action (Sub_into (Sub_into (Leaf_dynamic <opaque>)))))
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 2) (stabilizations_caused_by_var_changes 0)
     (stabilizations_skipped 2) (prunes_run 0) (branches_pruned 0))
    |}]
;;

let%expect_test "Wrap/Model_resetter" =
  let component =
    Bonsai.wrap
      ~default_model:()
      ~apply_action:(fun _context _ () () -> ())
      ~f:(fun (_ : unit Value.t) inject_outer ->
        let%sub (_, inject), inject_reset = Bonsai.with_model_resetter no_op_sm1 in
        let%arr inject_outer = inject_outer
        and inject_reset = inject_reset
        and inject = inject in
        inject_outer (), inject_reset, inject ())
      ()
  in
  let module Action = struct
    type t =
      | Wrap_outer
      | Model_reset
      | Inject_dynamic
    [@@deriving sexp_of]
  end
  in
  let handle =
    Handle.create
      (module struct
        type t = unit Effect.t * unit Effect.t * unit Effect.t
        type incoming = Action.t

        let view _ = ""

        let incoming (wrap_outer, model_reset, inject) = function
          | Action.Wrap_outer -> wrap_outer
          | Model_reset -> model_reset
          | Inject_dynamic -> inject
        ;;
      end)
      component
  in
  Handle.print_actions handle;
  Handle.print_stabilizations handle;
  (* wrap outer *)
  Handle.do_actions handle [ Wrap_outer ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action" (action (Wrap_outer <opaque>)))
    |}];
  (* model reset *)
  Handle.do_actions handle [ Model_reset ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action" (action (Wrap_inner Model_reset_outer)))
    |}];
  (* inner dynamic *)
  Handle.do_actions handle [ Inject_dynamic ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action"
     (action (Wrap_inner (Model_reset_inner (Leaf_dynamic <opaque>)))))
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 0) (stabilizations_caused_by_var_changes 0)
     (stabilizations_skipped 3) (prunes_run 0) (branches_pruned 0))
    |}];
  (* inner then outer *)
  Handle.do_actions handle [ Inject_dynamic; Wrap_outer ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action"
     (action (Wrap_inner (Model_reset_inner (Leaf_dynamic <opaque>)))))
    stabilized
    ("Processed action" (action (Wrap_outer <opaque>)))
    |}];
  (* reset then outer *)
  Handle.do_actions handle [ Model_reset; Wrap_outer ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action" (action (Wrap_inner Model_reset_outer)))
    stabilized
    ("Processed action" (action (Wrap_outer <opaque>)))
    |}]
;;

let%expect_test "Switch/Lazy" =
  let lazy_branch_var = Bonsai.Var.create false in
  let lazy_branch = Bonsai.Var.value lazy_branch_var in
  let component =
    match%sub lazy_branch with
    | false ->
      let%sub _, inject = no_op_sm0 in
      let%arr inject = inject in
      inject ()
    | true ->
      (Bonsai.lazy_ [@alert "-deprecated"])
        (lazy
          (let%sub _, inject = no_op_sm0 in
           let%arr inject = inject in
           inject ()))
  in
  let handle =
    Handle.create
      (module struct
        type t = unit Effect.t
        type incoming = unit

        let view _ = ""
        let incoming inject () = inject
      end)
      component
  in
  Handle.print_actions handle;
  Handle.print_stabilizations handle;
  (* In this case, we should go through the first switch branch and not hit the lazy
     case *)
  Handle.do_actions handle [ () ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action" (action (Switch 0 (Leaf_static <opaque>))))
    |}];
  Bonsai.Var.set lazy_branch_var true;
  Handle.recompute_view_until_stable handle;
  (* And alternatively, in this case, we should go through the second branch and hit
     the lazy case *)
  Handle.do_actions handle [ () ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action" (action (Switch 1 (Lazy (Leaf_static <opaque>)))))
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 0) (stabilizations_caused_by_var_changes 0)
     (stabilizations_skipped 2) (prunes_run 0) (branches_pruned 0))
    |}]
;;

let%expect_test "Assoc" =
  let input = Bonsai.Var.create (Int.Map.of_alist_exn [ 1, (); 2, () ]) in
  let component =
    Bonsai.assoc
      (module Int)
      (Bonsai.Var.value input)
      ~f:(fun _ _ ->
        let%sub _, inject = no_op_sm0 in
        let%arr inject = inject in
        inject ())
  in
  let module Action = struct
    type t = Entry of int [@@deriving sexp_of]
  end
  in
  let handle =
    Handle.create
      (module struct
        type t = unit Effect.t Int.Map.t
        type incoming = Action.t

        let view _ = ""
        let incoming m (Action.Entry i) = Map.find_exn m i
      end)
      component
  in
  Handle.print_actions handle;
  Handle.print_stabilizations handle;
  Handle.do_actions handle [ Entry 1; Entry 2 ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action" (action (Assoc 1 (Leaf_static <opaque>))))
    skipped stabilization
    ("Processed action" (action (Assoc 2 (Leaf_static <opaque>))))
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 0) (stabilizations_caused_by_var_changes 0)
     (stabilizations_skipped 2) (prunes_run 0) (branches_pruned 0))
    |}]
;;

let%expect_test "Assoc_on" =
  let input = Bonsai.Var.create (Int.Map.of_alist_exn [ 1, (); 2, () ]) in
  let component =
    Bonsai.Expert.assoc_on
      (module Int)
      (module Unit)
      ~get_model_key:(fun _ _ -> ())
      ~f:(fun _ _ -> Bonsai.state 0)
      (Bonsai.Var.value input)
  in
  let module Action = struct
    type t =
      | Entry of
          { key : int
          ; set_to : int
          }
    [@@deriving sexp_of]
  end
  in
  let handle =
    Handle.create
      (module struct
        type t = (int * (int -> unit Effect.t)) Int.Map.t
        type incoming = Action.t

        let view m =
          Map.to_alist m
          |> List.map ~f:(fun (i, (state, _inject)) -> i, state)
          |> [%sexp_of: (int * int) list]
          |> Sexp.to_string_hum
        ;;

        let incoming m (Action.Entry { key; set_to }) =
          let _state, inject = Map.find_exn m key in
          inject set_to
        ;;
      end)
      component
  in
  Handle.print_actions handle;
  Handle.do_actions handle [ Entry { key = 1; set_to = 1 } ];
  Handle.show handle;
  [%expect
    {|
    ("Processed action" (action (Assoc_on 1 () (Leaf_static <opaque>))))
    ((1 1) (2 1))
    |}];
  Handle.do_actions handle [ Entry { key = 2; set_to = 2 } ];
  Handle.show handle;
  [%expect
    {|
    ("Processed action" (action (Assoc_on 2 () (Leaf_static <opaque>))))
    ((1 2) (2 2))
    |}]
;;

let%expect_test "Dynamic actions applied across frames don't need extra stabilizations \
                 due to var setting between frames"
  =
  let component =
    let%sub (), inject_first = no_op_sm1 in
    let%sub (), inject_second = no_op_sm1 in
    let%arr inject_first = inject_first
    and inject_second = inject_second in
    inject_first (), inject_second ()
  in
  let module Action = struct
    type t =
      | First
      | Second
  end
  in
  let handle =
    Handle.create
      (module struct
        type t = unit Effect.t * unit Effect.t
        type incoming = Action.t

        let view _ = ""

        let incoming (first, second) = function
          | Action.First -> first
          | Second -> second
        ;;
      end)
      component
  in
  Handle.print_stabilizations handle;
  Handle.do_actions handle [ Second; First ];
  Handle.show handle;
  [%expect {|
    skipped stabilization
    skipped stabilization
    |}];
  Handle.do_actions handle [ Second ];
  Handle.show handle;
  [%expect {| skipped stabilization |}];
  (* Note: We don't need an additional stabilization here because stabilization happens
     between frames (i.e. as part of [Handle.show], after applying [Second] above). *)
  Handle.do_actions handle [ First ];
  Handle.show handle;
  [%expect {| skipped stabilization |}];
  Handle.show handle;
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 0) (stabilizations_caused_by_var_changes 0)
     (stabilizations_skipped 4) (prunes_run 0) (branches_pruned 0))
    |}]
;;

let%expect_test "A static action that affects a dynamic action forces a restabilization" =
  let component =
    let%sub input, inject_static =
      Bonsai.state_machine0
        ~default_model:0
        ~apply_action:(fun _context model () -> model + 1)
        ()
    in
    let%sub output, inject_dynamic = copying_sm1 input in
    let%arr output = output
    and inject_static = inject_static
    and inject_dynamic = inject_dynamic in
    output, inject_static (), inject_dynamic ()
  in
  let module Action = struct
    type t =
      | Static
      | Dynamic
  end
  in
  let handle =
    Handle.create
      (module struct
        type t = int * unit Effect.t * unit Effect.t
        type incoming = Action.t

        let view (int, _, _) = [%string "Model: %{int#Int}"]

        let incoming (_, static, dynamic) = function
          | Action.Static -> static
          | Dynamic -> dynamic
        ;;
      end)
      component
  in
  Handle.print_actions handle;
  Handle.print_stabilizations handle;
  Handle.show handle;
  [%expect {| Model: 0 |}];
  (* Applying the static action should increase the model of the first state machine to 1,
     meaning that applying the dynamic action should change the output to 2. *)
  Handle.do_actions handle [ Static; Dynamic ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    ("Processed action" (action (Sub_from (Leaf_static <opaque>))))
    stabilized
    ("Processed action" (action (Sub_into (Leaf_dynamic <opaque>))))
    Model: 1
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 1) (stabilizations_caused_by_var_changes 0)
     (stabilizations_skipped 1) (prunes_run 0) (branches_pruned 0))
    |}]
;;

(* These tests verify interactions between mutable bits of incremental/bonsai, which can
   be tricky to get correct. In some cases, the code written is not idiomatic Bonsai (or,
   even idiomatic Bonsai test. This is alright, because we still want to handle people
   miusing/breaking into mutable APIs. *)
let%expect_test "state_machine1 depending on a Bonsai var behaves properly" =
  let var = Bonsai.Var.create 0 in
  let component = copying_sm1 (Bonsai.Var.value var) in
  let handle =
    Handle.create
      (module struct
        type t = int * (unit -> unit Effect.t)
        type incoming = unit

        let view (int, _) = [%string "Model: %{int#Int}"]
        let incoming (_, inject) () = inject ()
      end)
      component
  in
  Handle.print_stabilizations handle;
  Handle.show handle;
  [%expect {| Model: 0 |}];
  Bonsai.Var.set var 2;
  Handle.do_actions handle [ () ];
  Handle.show handle;
  [%expect {|
    stabilized
    Model: 2
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 1) (stabilizations_caused_by_var_changes 1)
     (stabilizations_skipped 0) (prunes_run 0) (branches_pruned 0))
    |}]
;;

let%expect_test "state doesn't require stabilizing when a var is set" =
  let var = Bonsai.Var.create 0 in
  let component = Bonsai.state 0 in
  let handle =
    Handle.create
      (module struct
        type t = int * (int -> unit Effect.t)
        type incoming = int

        let view (int, _) = [%string "Model: %{int#Int}"]
        let incoming (_, inject) = inject
      end)
      component
  in
  Handle.print_stabilizations handle;
  Handle.show handle;
  [%expect {| Model: 0 |}];
  Bonsai.Var.set var 2;
  Handle.do_actions handle [ 5 ];
  Handle.show handle;
  [%expect {|
    skipped stabilization
    Model: 5
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 0) (stabilizations_caused_by_var_changes 0)
     (stabilizations_skipped 1) (prunes_run 0) (branches_pruned 0))
    |}]
;;

let%expect_test "state_machine1 depending on an Incr.compute with an Incr var behaves \
                 properly"
  =
  let incr_var = Incr.Var.create 0 in
  let component =
    let%sub value =
      Bonsai.Var.create ()
      |> Bonsai.Var.value
      |> Bonsai.Incr.compute ~f:(fun _ -> Incr.Var.watch incr_var)
    in
    copying_sm1 value
  in
  let handle =
    Handle.create
      (module struct
        type t = int * (unit -> unit Effect.t)
        type incoming = unit

        let view (int, _) = [%string "Model: %{int#Int}"]
        let incoming (_, inject) () = inject ()
      end)
      component
  in
  Handle.print_stabilizations handle;
  Handle.show handle;
  [%expect {| Model: 0 |}];
  Incr.Var.set incr_var 2;
  Handle.do_actions handle [ () ];
  Handle.show handle;
  [%expect {|
    stabilized
    Model: 2
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 1) (stabilizations_caused_by_var_changes 1)
     (stabilizations_skipped 0) (prunes_run 0) (branches_pruned 0))
    |}]
;;

let%expect_test "state_machine1 that schedules an action which sets an upstream var \
                 behaves properly"
  =
  let incr_var = Incr.Var.create 0 in
  let component =
    let%sub value =
      Bonsai.Var.create ()
      |> Bonsai.Var.value
      |> Bonsai.Incr.compute ~f:(fun _ -> Incr.Var.watch incr_var)
    in
    let%sub state, inject = copying_sm1 value in
    let%sub (), inject_mutation =
      Bonsai.state_machine0
        ~default_model:()
        ~apply_action:(fun _context () value -> Incr.Var.set incr_var value)
        ()
    in
    let%arr state = state
    and inject = inject
    and inject_mutation = inject_mutation in
    state, inject, inject_mutation
  in
  let handle =
    Handle.create
      (module struct
        type t = int * (unit -> unit Effect.t) * (int -> unit Effect.t)

        type incoming =
          [ `Set_input of int
          | `Copy_input
          ]

        let view (int, _, _) = [%string "Model: %{int#Int}"]

        let incoming (_, inject_copy, inject_mutate) = function
          | `Copy_input -> inject_copy ()
          | `Set_input x -> inject_mutate x
        ;;
      end)
      component
  in
  Handle.print_stabilizations handle;
  Handle.show handle;
  [%expect {| Model: 0 |}];
  (* It handles when actions with side-effects are applied in the same frame *)
  Handle.do_actions handle [ `Set_input 1; `Copy_input ];
  Handle.show handle;
  [%expect {|
    skipped stabilization
    stabilized
    Model: 1
    |}];
  (* It handles when actions are applied across multiple frames *)
  Handle.do_actions handle [ `Set_input 2 ];
  Handle.show handle;
  [%expect {|
    skipped stabilization
    Model: 1
    |}];
  Handle.do_actions handle [ `Copy_input ];
  Handle.show handle;
  [%expect {|
    skipped stabilization
    Model: 2
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 1) (stabilizations_caused_by_var_changes 1)
     (stabilizations_skipped 3) (prunes_run 0) (branches_pruned 0))
    |}]
;;

(* This test demonstrates a case where we don't stabilize even though technically we
   should. This is an example of a misuse of the [Incr.Expert.Node] API where impurity is
   leaking to the user. While we could technically guard against this, users breaking
   Incremental's abstractions do so at their own risk. *)
let%expect_test "state_machine1 depending on Incr.Expert.Node.t that breaks abstraction \
                 doesn't stabilize when marked stale"
  =
  let counter = ref 0 in
  let expert_node =
    Incr.Expert.Node.create (fun () ->
      print_endline "Expert body fired";
      let value = !counter in
      incr counter;
      value)
  in
  let component =
    let%sub value =
      Bonsai.Var.create ()
      |> Bonsai.Var.value
      |> Bonsai.Incr.compute ~f:(fun _ -> Incr.Expert.Node.watch expert_node)
    in
    let%sub model, inject_copy = copying_sm1 value in
    let%sub (), inject_stale =
      Bonsai.state_machine0
        ~default_model:()
        ~apply_action:(fun _context () () -> Incr.Expert.Node.make_stale expert_node)
        ()
    in
    let%arr model = model
    and inject_copy = inject_copy
    and inject_stale = inject_stale in
    model, inject_copy, inject_stale
  in
  let handle =
    Handle.create
      (module struct
        type t = int * (unit -> unit Effect.t) * (unit -> unit Effect.t)

        type incoming =
          [ `Make_stale
          | `Copy_input
          ]

        let view (int, _, _) = [%string "Model: %{int#Int}"]

        let incoming (_, inject_copy, inject_stale) = function
          | `Make_stale -> inject_stale ()
          | `Copy_input -> inject_copy ()
        ;;
      end)
      component
  in
  Handle.print_stabilizations handle;
  Handle.show handle;
  [%expect {|
    Expert body fired
    Model: 0
    |}];
  (* Actions with side-effects are applied in the same frame *)
  Handle.do_actions handle [ `Make_stale; `Copy_input ];
  Handle.show handle;
  [%expect
    {|
    skipped stabilization
    skipped stabilization
    Expert body fired
    Model: 0
    |}];
  (* Actions are applied across multiple frames *)
  Handle.do_actions handle [ `Make_stale ];
  Handle.show handle;
  [%expect {|
    skipped stabilization
    Expert body fired
    Model: 0
    |}];
  Handle.do_actions handle [ `Copy_input ];
  Handle.show handle;
  [%expect {|
    skipped stabilization
    Model: 2
    |}];
  Handle.print_stabilization_tracker_stats handle;
  [%expect
    {|
    ((stabilizations_before_actions 0) (stabilizations_caused_by_var_changes 0)
     (stabilizations_skipped 4) (prunes_run 0) (branches_pruned 0))
    |}]
;;

let%test_module "pruning" =
  (module struct
    let trigger_prune_with action =
      (* We do [2 * num_generations_for_pruning] because pruning only happens every
         [num_generations_for_testing] and requires the path to not have received an action
         for the last [num_generations_for_testing] stabilizations. *)
      List.init
        (2 * Bonsai.Private.Stabilization_tracker.For_testing.num_generations_for_pruning)
        ~f:(fun _ -> action)
    ;;

    let sm1_requiring_stabilization =
      let%sub _, inject_first = no_op_sm0 in
      let%sub _, inject_second = no_op_sm1 in
      let%arr inject_first = inject_first
      and inject_second = inject_second in
      Ui_effect.Many [ inject_first (); inject_second () ]
    ;;

    let run_assoc_test assoc_impl =
      let input = Bonsai.Var.create (Int.Map.of_alist_exn [ 1, (); 2, () ]) in
      let component =
        assoc_impl (Bonsai.Var.value input) ~f:(fun _ _ -> sm1_requiring_stabilization)
      in
      let module Action = struct
        type t = Entry of int [@@deriving sexp_of]
      end
      in
      let handle =
        Handle.create
          (module struct
            type t = unit Effect.t Int.Map.t
            type incoming = Action.t

            let view _ = ""
            let incoming m (Action.Entry i) = Map.find_exn m i
          end)
          component
      in
      Handle.do_actions handle [ Entry 2; Entry 1 ];
      Handle.recompute_view_until_stable handle;
      Handle.do_actions handle (trigger_prune_with (Action.Entry 1));
      Handle.recompute_view_until_stable handle;
      print_endline "After pruning:";
      Handle.print_stabilization_tracker_stats handle;
      (* Note: at this point, we have pruned the branch corresponding to the key [2] in the
         assoc. We can still deliver actions to this path again without issues. *)
      Handle.do_actions handle [ Entry 2 ];
      Handle.recompute_view_until_stable handle;
      print_endline "After pruning + action delivery:";
      Handle.print_stabilization_tracker_stats handle
    ;;

    let%expect_test "old assoc branches are pruned after not receiving actions for a \
                     long time"
      =
      run_assoc_test (Bonsai.assoc (module Int));
      [%expect
        {|
        After pruning:
        ((stabilizations_before_actions 5402)
         (stabilizations_caused_by_var_changes 0) (stabilizations_skipped 5402)
         (prunes_run 2) (branches_pruned 1))
        After pruning + action delivery:
        ((stabilizations_before_actions 5403)
         (stabilizations_caused_by_var_changes 0) (stabilizations_skipped 5403)
         (prunes_run 2) (branches_pruned 1))
        |}]
    ;;

    let%expect_test "old assoc_on branches are pruned after not receiving actions for a \
                     long time"
      =
      run_assoc_test
        (Bonsai.Expert.assoc_on
           (module Int)
           (module Unit)
           ~get_model_key:(fun _ () -> ()));
      [%expect
        {|
        After pruning:
        ((stabilizations_before_actions 5402)
         (stabilizations_caused_by_var_changes 0) (stabilizations_skipped 5402)
         (prunes_run 2) (branches_pruned 1))
        After pruning + action delivery:
        ((stabilizations_before_actions 5403)
         (stabilizations_caused_by_var_changes 0) (stabilizations_skipped 5403)
         (prunes_run 2) (branches_pruned 1))
        |}]
    ;;

    let%expect_test "old switch branches are pruned after not receiving actions for a \
                     long time"
      =
      let input = Bonsai.Var.create true in
      let component =
        match%sub Bonsai.Var.value input with
        | true -> sm1_requiring_stabilization
        | false -> sm1_requiring_stabilization
      in
      let handle =
        Handle.create
          (module struct
            type t = unit Effect.t
            type incoming = unit

            let view _ = ""
            let incoming e () = e
          end)
          component
      in
      Handle.do_actions handle [ () ];
      Handle.recompute_view_until_stable handle;
      Bonsai.Var.set input false;
      Handle.recompute_view_until_stable handle;
      Handle.do_actions handle (trigger_prune_with ());
      (* Handle.do_actions handle (trigger_prune_with ()); *)
      Handle.recompute_view_until_stable handle;
      print_endline "After pruning:";
      Handle.print_stabilization_tracker_stats handle;
      (* Note: at this point, we have pruned the branch corresponding to the key [2] in the
         assoc. We can still deliver actions to this path again without issues. *)
      Bonsai.Var.set input true;
      Handle.recompute_view_until_stable handle;
      Handle.do_actions handle [ () ];
      Handle.recompute_view_until_stable handle;
      print_endline "After pruning + action delivery:";
      Handle.print_stabilization_tracker_stats handle;
      [%expect
        {|
        After pruning:
        ((stabilizations_before_actions 5401)
         (stabilizations_caused_by_var_changes 0) (stabilizations_skipped 5401)
         (prunes_run 2) (branches_pruned 1))
        After pruning + action delivery:
        ((stabilizations_before_actions 5402)
         (stabilizations_caused_by_var_changes 0) (stabilizations_skipped 5402)
         (prunes_run 2) (branches_pruned 1))
        |}]
    ;;
  end)
;;

let%test_module "the optimization takes effect" =
  (module struct
    let%expect_test "state_machine1 depending on state_machine1" =
      let component =
        let%sub _, inject1 = no_op_sm1 in
        let%sub _, inject2 = no_op_sm1 in
        let%arr inject1 = inject1
        and inject2 = inject2 in
        inject1 (), inject2 ()
      in
      let handle =
        Handle.create
          (module struct
            type t = unit Effect.t * unit Effect.t

            type incoming =
              [ `Upstream
              | `Downstream
              ]

            let view _ = ""

            let incoming (i1, i2) = function
              | `Upstream -> i1
              | `Downstream -> i2
            ;;
          end)
          component
      in
      Handle.print_actions handle;
      Handle.print_stabilizations handle;
      (* In this case, we do need a stabilization between actions *)
      Handle.do_actions handle [ `Upstream; `Downstream ];
      Handle.show handle;
      [%expect
        {|
        skipped stabilization
        ("Processed action" (action (Sub_from (Leaf_dynamic <opaque>))))
        stabilized
        ("Processed action" (action (Sub_into (Leaf_dynamic <opaque>))))
        |}];
      (* Whereas in this case, we don't need a stabilization between actions *)
      Handle.do_actions handle [ `Downstream; `Upstream ];
      Handle.show handle;
      [%expect
        {|
        skipped stabilization
        ("Processed action" (action (Sub_into (Leaf_dynamic <opaque>))))
        skipped stabilization
        ("Processed action" (action (Sub_from (Leaf_dynamic <opaque>))))
        |}];
      (* If we schedule two downstream actions, we don't need a stabilization between them
      *)
      Handle.do_actions handle [ `Downstream; `Downstream ];
      Handle.show handle;
      [%expect
        {|
        skipped stabilization
        ("Processed action" (action (Sub_into (Leaf_dynamic <opaque>))))
        skipped stabilization
        ("Processed action" (action (Sub_into (Leaf_dynamic <opaque>))))
        |}];
      Handle.print_stabilization_tracker_stats handle;
      [%expect
        {|
        ((stabilizations_before_actions 1) (stabilizations_caused_by_var_changes 0)
         (stabilizations_skipped 5) (prunes_run 0) (branches_pruned 0))
        |}]
    ;;

    let%expect_test "wrap that depends on a state_machine1" =
      let component =
        let%sub _, inject_sm1 = no_op_sm1 in
        let%sub inject_wrap_inner, inject_wrap_outer =
          Bonsai.wrap
            ~default_model:()
            ~apply_action:(fun _context _result _model _action -> ())
            ~f:(fun _model inject_outer ->
              let%sub _, inject_inner = no_op_sm1 in
              let%arr inject_inner = inject_inner
              and inject_outer = inject_outer in
              inject_inner, inject_outer)
            ()
        in
        let%arr inject_sm1 = inject_sm1
        and inject_wrap_inner = inject_wrap_inner
        and inject_wrap_outer = inject_wrap_outer in
        inject_sm1 (), inject_wrap_inner (), inject_wrap_outer ()
      in
      let handle =
        Handle.create
          (module struct
            type t = unit Effect.t * unit Effect.t * unit Effect.t

            type incoming =
              [ `Sm1
              | `Wrap_inner
              | `Wrap_outer
              ]

            let view _ = ""

            let incoming (sm1, inner, outer) = function
              | `Sm1 -> sm1
              | `Wrap_inner -> inner
              | `Wrap_outer -> outer
            ;;
          end)
          component
      in
      Handle.print_stabilizations handle;
      Handle.show handle;
      let actions = [ `Sm1; `Wrap_inner; `Wrap_outer ] in
      List.cartesian_product actions actions
      |> List.iter ~f:(fun (i, j) ->
           print_s
             [%message
               "Applying:"
                 ~_:(i : [ `Sm1 | `Wrap_inner | `Wrap_outer ])
                 ~_:(j : [ `Sm1 | `Wrap_inner | `Wrap_outer ])];
           Handle.do_actions handle [ i; j ];
           Handle.show handle);
      (* Here, we expect a test to stabilize before its second action if and only if
         the second action isn't [`Sm1]. *)
      [%expect
        {|
        (Applying: Sm1 Sm1)
        skipped stabilization
        skipped stabilization

        (Applying: Sm1 Wrap_inner)
        skipped stabilization
        stabilized

        (Applying: Sm1 Wrap_outer)
        skipped stabilization
        stabilized

        (Applying: Wrap_inner Sm1)
        skipped stabilization
        skipped stabilization

        (Applying: Wrap_inner Wrap_inner)
        skipped stabilization
        skipped stabilization

        (Applying: Wrap_inner Wrap_outer)
        skipped stabilization
        stabilized

        (Applying: Wrap_outer Sm1)
        skipped stabilization
        skipped stabilization

        (Applying: Wrap_outer Wrap_inner)
        skipped stabilization
        stabilized

        (Applying: Wrap_outer Wrap_outer)
        skipped stabilization
        stabilized
        |}];
      Handle.do_actions handle [ `Wrap_inner; `Wrap_inner; `Wrap_inner ];
      (* Chaining many of these wrap inners does not require stabilization *)
      Handle.show handle;
      [%expect
        {|
        skipped stabilization
        skipped stabilization
        skipped stabilization
        |}];
      Handle.do_actions handle [ `Wrap_outer; `Wrap_outer; `Wrap_outer ];
      (* Chaining many of these wrap outers together continues to require stabilization *)
      Handle.show handle;
      [%expect
        {|
        skipped stabilization
        stabilized
        stabilized
        |}]
    ;;
  end)
;;

let%test_module "interesting action delivery cases" =
  (module struct
    let%expect_test "dynamic action being delivered to a switch that requires \
                     stabilization"
      =
      let var = Bonsai.Var.create false in
      let component =
        let%sub input, set_input = Bonsai.state 0 in
        match%sub Bonsai.Var.value var with
        | false ->
          let%sub _state, set_state = Bonsai.state () in
          let%arr set_state = set_state
          and set_input = set_input in
          0, set_state, set_input
        | true ->
          let%sub input, copy_input = copying_sm1 input in
          let%arr input = input
          and copy_input = copy_input
          and set_input = set_input in
          input, copy_input, set_input
      in
      let handle =
        Handle.create
          (module struct
            type t = int * (unit -> unit Effect.t) * (int -> unit Effect.t)

            type incoming =
              [ `Set of int
              | `Copy
              ]

            let view (i, _, _) = Int.to_string i

            let incoming (_, copy, set) = function
              | `Set i -> set i
              | `Copy -> copy ()
            ;;
          end)
          component
      in
      (* We deliver an action to the true branch, and then switch to the false branch and
         perform both a set and a copy in the same [do_actions]. The copy should require a
         stabilization and correctly observe the set value. *)
      Handle.show handle;
      [%expect {| 0 |}];
      Handle.do_actions handle [ `Copy ];
      Bonsai.Var.set var true;
      Handle.recompute_view_until_stable handle;
      Handle.do_actions handle [ `Set 2; `Copy ];
      Handle.show handle;
      [%expect {| 2 |}]
    ;;

    let%expect_test "dynamic action being delivered to an assoc that requires \
                     stabilization"
      =
      let component =
        let%sub input, set_input = Bonsai.state 0 in
        let%sub m =
          Bonsai.assoc
            (module Int)
            (opaque_const_value (Int.Map.of_alist_exn [ 1, (); 2, () ]))
            ~f:(fun _key _data -> copying_sm1 input)
        in
        let%arr m = m
        and set_input = set_input in
        m, set_input
      in
      let handle =
        Handle.create
          (module struct
            type t = (int * (unit -> unit Effect.t)) Int.Map.t * (int -> unit Effect.t)

            type incoming =
              [ `Copy of int
              | `Set of int
              ]

            let view (m, _) =
              Map.to_alist m
              |> List.map ~f:(fun (key, (state, _set)) -> key, state)
              |> [%sexp_of: (int * int) list]
              |> Sexp.to_string_hum
            ;;

            let incoming (m, set) = function
              | `Copy i ->
                let _, copy = Map.find_exn m i in
                copy ()
              | `Set i -> set i
            ;;
          end)
          component
      in
      (* We deliver an action to the 1-key, and then another one to the 2-key, with sets
         prior to both. Both copies should notice the need for a stabilization to
         correctly observe the set value. *)
      Handle.show handle;
      [%expect {| ((1 0) (2 0)) |}];
      Handle.do_actions handle [ `Set 2; `Copy 1; `Set 3; `Copy 2 ];
      Handle.show handle;
      [%expect {| ((1 2) (2 3)) |}]
    ;;

    let%expect_test "dynamic action being delivered to an assoc that requires \
                     stabilization"
      =
      let component =
        let%sub input, set_input = Bonsai.state 0 in
        let%sub m =
          Bonsai.Expert.assoc_on
            (module Int)
            (module Unit)
            ~get_model_key:(fun _ _ -> ())
            (opaque_const_value (Int.Map.of_alist_exn [ 1, (); 2, () ]))
            ~f:(fun _key _data -> copying_sm1 input)
        in
        let%arr m = m
        and set_input = set_input in
        m, set_input
      in
      let handle =
        Handle.create
          (module struct
            type t = (int * (unit -> unit Effect.t)) Int.Map.t * (int -> unit Effect.t)

            type incoming =
              [ `Copy of int
              | `Set of int
              ]

            let view (m, _) =
              Map.to_alist m
              |> List.map ~f:(fun (key, (state, _set)) -> key, state)
              |> [%sexp_of: (int * int) list]
              |> Sexp.to_string_hum
            ;;

            let incoming (m, set) = function
              | `Copy i ->
                let _, copy = Map.find_exn m i in
                copy ()
              | `Set i -> set i
            ;;
          end)
          component
      in
      (* We deliver an action to the 1-key, and then another one to the 2-key, with sets
         prior to both. Both copies should notice the need for a stabilization to
         correctly observe the set value. *)
      Handle.show handle;
      [%expect {| ((1 0) (2 0)) |}];
      Handle.do_actions handle [ `Set 2; `Copy 1 ];
      Handle.show handle;
      [%expect {| ((1 2) (2 2)) |}];
      Handle.do_actions handle [ `Set 3; `Copy 2 ];
      Handle.show handle;
      [%expect {| ((1 3) (2 3)) |}]
    ;;

    (* This test case exercises the logic for a model reset outer action being followed by
       model reset inner action within the same frame. *)
    let%expect_test "a model reset containing a wrap" =
      let component =
        Bonsai.with_model_resetter
          (Bonsai.wrap
             ~default_model:0
             ~apply_action:(fun _context (result, _) _model -> function
               | `Set i -> i
               | `Copy -> result)
             ~f:(fun model inject_outer ->
               let%arr model = model
               and inject_outer = inject_outer in
               model, inject_outer)
             ())
      in
      let handle =
        Handle.create
          (module struct
            type t = (int * ([ `Set of int | `Copy ] -> unit Effect.t)) * unit Effect.t

            type incoming =
              [ `Set of int
              | `Copy
              | `Reset
              ]

            let view ((state, _), _) = Int.to_string state

            let incoming ((_, inject), reset) = function
              | `Set i -> inject (`Set i)
              | `Copy -> inject `Copy
              | `Reset -> reset
            ;;
          end)
          component
      in
      Handle.show handle;
      [%expect {| 0 |}];
      Handle.do_actions handle [ `Set 10 ];
      Handle.show handle;
      [%expect {| 10 |}];
      (* Resetting and copying in the same frame should copy the new value of 0, instead
         of the old value of 10. *)
      Handle.do_actions handle [ `Reset; `Copy ];
      Handle.show handle;
      [%expect {| 0 |}]
    ;;

    let%expect_test "consecutive wrap inner actions don't require stabilization, even if \
                     they call inject_outer"
      =
      (* This test demonstrates that successive [Wrap_inner] actions don't require
         stabiliztaion. The only way that [Wrap_inner] could influence its own inputs is
         by calling the supplied [inject_outer] input, which is what this test does. *)
      let component =
        Bonsai.wrap
          ~default_model:0
          ~apply_action:(fun _context (result, _) _model -> function
            | `Set i -> i
            | `Copy -> result)
          ~f:(fun model inject_outer ->
            let%sub (), inject_inner =
              Bonsai.state_machine1
                ~default_model:()
                ~apply_action:(fun context inject_outer _model action ->
                  match inject_outer with
                  | Inactive -> raise_s [%message "BUG"]
                  | Active inject_outer ->
                    let schedule_event =
                      Bonsai.Apply_action_context.schedule_event context
                    in
                    schedule_event (inject_outer action))
                inject_outer
            in
            let%arr model = model
            and inject_inner = inject_inner in
            model, inject_inner)
          ()
      in
      let handle =
        Handle.create
          (module struct
            type t = int * ([ `Set of int | `Copy ] -> unit Effect.t)

            type incoming =
              [ `Set of int
              | `Copy
              ]

            let view (state, _) = Int.to_string state

            let incoming (_, inject) = function
              | `Set i -> inject (`Set i)
              | `Copy -> inject `Copy
            ;;
          end)
          component
      in
      Handle.print_stabilizations handle;
      Handle.print_actions handle;
      Handle.show handle;
      [%expect {| 0 |}];
      (* Setting the model to 5 and then copying it in the same frame should cause the
         model to (correctly) update to 5 *)
      Handle.do_actions handle [ `Set 5; `Copy ];
      Handle.show handle;
      (* The actions below demonstrate why this action is safe in the first place: even
         though we call the outer injection function from the body of the wrap, that
         results in a [Wrap_outer] action being generated. We require stabilization
         between [Wrap_inner] and [Wrap_outer] events, so the data is all up-to-date. *)
      [%expect
        {|
        skipped stabilization
        ("Processed action" (action (Wrap_inner (Leaf_dynamic <opaque>))))
        skipped stabilization
        ("Processed action" (action (Wrap_inner (Leaf_dynamic <opaque>))))
        stabilized
        ("Processed action" (action (Wrap_outer <opaque>)))
        stabilized
        ("Processed action" (action (Wrap_outer <opaque>)))
        5
        |}]
    ;;
  end)
;;